/*
 * Copyright (c) 1990 Regents of the University of Michigan.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of Michigan at Ann Arbor. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/syslog.h>
#include <sys/wait.h>

typedef struct netauth {
	char	*na_dn;
	char	**na_nets;
} Netauth;

static Netauth	*nets;
static int	numnets;

read_network_file( fname )
    char	*fname;
{
	int	len;
	char	*s;
	char	*dn, *networks;
	FILE	*fp;
	char	buf[BUFSIZ];

	if ( (fp = fopen( fname, "r" )) == NULL ) {
		syslog( LOG_INFO, "Cannot open network auth file (%s)",
		    fname );
		return( -1 );
	}

	while ( fgets( buf, sizeof(buf), fp ) != NULL ) {
		/* comment */
		if ( buf[0] == '#' )
			continue;

		/* DN */
		if ( (s = strchr( buf, '\n' )) != NULL )
			*s = '\0';
		dn = strdup( buf );

		if ( fgets( buf, sizeof(buf), fp ) == NULL ) {
			syslog( LOG_INFO, "Bad network auth file format (%s)",
			    fname );
			return( -1 );
		}

		/* network list */
		if ( (s = strchr( buf, '\n' )) != NULL )
			*s = '\0';
		networks = strdup( buf );

		add_network( dn, networks );
	}

	return( 0 );
}

add_network( dn, netlist )
    char	*dn;
    char	*netlist;
{
	char	*s, *n;
	char	**netarray;
	int	i;

	if ( nets == NULL ) {
		nets = (Netauth *) calloc( 1, sizeof(Netauth) );
	} else {
		nets = (Netauth *) realloc( nets, (numnets + 1) *
		    sizeof(Netauth) );
	}
	numnets++;

	if ( strcmp( dn, "NULLDN" ) == 0 ) {
		nets[numnets - 1].na_dn = NULL;
	} else {
		nets[numnets - 1].na_dn = dn;
	}
	nets[numnets - 1].na_nets = NULL;

	if ( strncasecmp( netlist, "default", 7 ) == 0 ) {
		return( 0 );
	}

	netarray = NULL;
	i = 0;
	s = netlist;
	while ( s != NULL && *s ) {
		while ( *s == ' ' || *s == '\t' )
			s++;

		if ( (n = strchr( s, ' ' )) != NULL || (n = strchr( s, '\t' ))
		    != NULL ) {
			*n = '\0';
		}

		if ( inet_addr( s ) == -1 ) {
			syslog( LOG_INFO, "Bad network number (%s)\n", s );
			s = ++n;
			continue;
		}

		if ( i == 0 ) {
			netarray = (char **) malloc( 2 * sizeof(char *) );
		} else {
			netarray = (char **) realloc( netarray, (i + 2)
			    * sizeof(char *) );
		}

		netarray[i++] = s;
		netarray[i] = NULL;

		s = n ? ++n : NULL;
	}

	nets[numnets - 1].na_nets = netarray;

	return( 0 );
}

char *
network2dn( net )
    char	*net;
{
	int	i, j;
	Netauth	*default_net;

	default_net = NULL;
	for ( i = 0; i < numnets; i++ ) {
		if ( nets[i].na_nets == NULL ) {
			default_net = &nets[i];
			continue;
		}

		for ( j = 0; nets[i].na_nets[j] != NULL; j++ ) {
			if ( strncmp( nets[i].na_nets[j], net,
			    strlen( nets[i].na_nets[j] ) ) == 0 ) {
				return( nets[i].na_dn );
			}
		}
	}

	return( default_net ? default_net->na_dn : NULL );
}
