/*
 *  Copyright (c) 1990 Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  synchronous.c - synchronous interface to the ldap protocol
 */

#ifndef lint 
static char copyright[] = "@(#) Copyright (c) 1990 Regents of the University of Michigan.\nAll rights reserved.\n";
#endif

#include "lber.h"
#include "ldap.h"
#include <stdio.h>
#include <string.h>
#ifdef MACOS
#include <stdlib.h>
#include "macos.h"
#else /* MACOS */
#ifdef DOS
#include "msdos.h"
#else /* DOS */
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#endif /* DOS */
#endif /* MACOS */

ldap_bind_s( ld, who, cred, method )
LDAP	*ld;
char	*who;
char	*cred;
int	method;
{
	return( ldap_bind( ld, who, cred, method ) );
}

ldap_unbind_s( ld )
LDAP	*ld;
{
	return( ldap_unbind( ld ) );
}

ldap_add_s( ld, dn, attrs )
LDAP		*ld;
char		*dn;
LDAPMod		**attrs;
{
	int		msgid;
	LDAPMessage	*res;

	if ( (msgid = ldap_add( ld, dn, attrs )) == -1 )
		return( ld->ld_errno );

	if ( ldap_result( ld, msgid, 1, (struct timeval *) NULL, &res ) == -1 )
		return( ld->ld_errno );

	return( ldap_result2error( ld, res, 1 ) );
}

ldap_delete_s( ld, dn )
LDAP		*ld;
char		*dn;
{
	int		msgid;
	LDAPMessage	*res;

	if ( (msgid = ldap_delete( ld, dn )) == -1 )
		return( ld->ld_errno );

	if ( ldap_result( ld, msgid, 1, (struct timeval *) NULL, &res ) == -1 )
		return( ld->ld_errno );

	return( ldap_result2error( ld, res, 1 ) );
}

ldap_modify_s( ld, dn, mods )
LDAP		*ld;
char		*dn;
LDAPMod		**mods;
{
	int		msgid;
	LDAPMessage	*res;

	if ( (msgid = ldap_modify( ld, dn, mods )) == -1 )
		return( ld->ld_errno );

	if ( ldap_result( ld, msgid, 1, (struct timeval *) NULL, &res ) == -1 )
		return( ld->ld_errno );

	return( ldap_result2error( ld, res, 1 ) );
}

ldap_modrdn_s( ld, dn, newrdn )
LDAP		*ld;
char		*dn;
char		*newrdn;
{
	int		msgid;
	LDAPMessage	*res;

	if ( (msgid = ldap_modrdn( ld, dn, newrdn )) == -1 )
		return( ld->ld_errno );

	if ( ldap_result( ld, msgid, 1, (struct timeval *) NULL, &res ) == -1 )
		return( ld->ld_errno );

	return( ldap_result2error( ld, res, 1 ) );
}

ldap_search_st( ld, base, scope, filter, attrs, attrsonly, timeout, res )
LDAP		*ld;
char		*base;
int		scope;
char		*filter;
char		**attrs;
int		attrsonly;
struct timeval	*timeout;
LDAPMessage	**res;
{
	int	msgid;

	if ( (msgid = ldap_search( ld, base, scope, filter, attrs, attrsonly ))
	    == -1 )
		return( ld->ld_errno );

	if ( ldap_result( ld, msgid, 1, timeout, res ) == -1 )
		return( ld->ld_errno );

	if ( ld->ld_errno == LDAP_TIMEOUT ) {
		(void) ldap_abandon( ld, msgid );
		return( LDAP_TIMEOUT );
	}

	return( ldap_result2error( ld, *res, 0 ) );
}

ldap_search_s( ld, base, scope, filter, attrs, attrsonly, res )
LDAP		*ld;
char		*base;
int		scope;
char		*filter;
char		**attrs;
int		attrsonly;
LDAPMessage	**res;
{
	int	msgid;

	if ( (msgid = ldap_search( ld, base, scope, filter, attrs, attrsonly ))
	    == -1 )
		return( ld->ld_errno );

	if ( ldap_result( ld, msgid, 1, (struct timeval *) NULL, res ) == -1 )
		return( ld->ld_errno );

	return( ldap_result2error( ld, *res, 0 ) );
}

ldap_compare_s( ld, dn, attr, value )
LDAP		*ld;
char		*dn;
char		*attr;
char		*value;
{
	int		msgid;
	LDAPMessage	*res;

	if ( (msgid = ldap_compare( ld, dn, attr, value )) == -1 )
		return( ld->ld_errno );

	if ( ldap_result( ld, msgid, 1, (struct timeval *) NULL, &res ) == -1 )
		return( ld->ld_errno );

	return( ldap_result2error( ld, res, 1 ) );
}
